@@ -90,6 +90,9 @@ SettingsUtils.get().then((items) => {
             if (status && status.purpose) {
                 loadPurpose();
             }
+        }, (err) => {
+            // No warning, can also happen when port isn't connected for unsupported OS
+            console.log("Failed to get subsystem status for purpose", err);
         });
     }
 });
@@ -338,8 +341,9 @@ function setPlayerActive(player) {
     // a website might have set Media Sessions metadata prior to playing
     // and then we would have ignored the metadata signal because there was no player
     sendMessage("mpris", "playing", {
-        mediaSrc: player.src,
+        mediaSrc: player.currentSrc || player.src,
         pageTitle: document.title,
+        poster: player.poster,
         duration: player.duration,
         currentTime: player.currentTime,
         playbackRate: player.playbackRate,
@@ -586,6 +590,11 @@ function loadMpris() {
                             return; // continue
                         }
 
+                        // If the player got temporarily added by us, don't consider it gone
+                        if (player.dataset.pbiPausedForDomRemoval === "true") {
+                            return;
+                        }
+
                         sendPlayerGone();
                         return;
                     }
@@ -599,10 +608,11 @@ function loadMpris() {
         subtree: true
     });
 
-    window.addEventListener("beforeunload", function () {
+    window.addEventListener("pagehide", function () {
         // about to navigate to a different page, tell our extension that the player will be gone shortly
         // we listen for tab closed in the extension but we don't for navigating away as URL change doesn't
-        // neccesarily mean a navigation but beforeunload *should* be the thing we want
+        // neccesarily mean a navigation.
+        // NOTE beforeunload is not emitted for iframes!
         sendPlayerGone();
     });
 
@@ -788,12 +798,13 @@ function loadMediaSessionsShim() {
         // so it continues playing :-)
         const addPlayerToDomEvadingAutoPlayBlocking = `
             player.registerInDom = () => {
-                player.pausedBecauseOfDomRemoval = true;
+                // Needs to be dataset so it's accessible from mutation observer on webpage
+                player.dataset.pbiPausedForDomRemoval = "true";
                 player.removeEventListener("play", player.registerInDom);
 
                 // If it is already in DOM by the time it starts playing, we don't need to do anything
                 if (document.body && document.body.contains(player)) {
-                    delete player.pausedBecauseOfDomRemoval;
+                    delete player.dataset.pbiPausedForDomRemoval;
                     player.removeEventListener("pause", player.replayAfterRemoval);
                 } else {
                     (document.head || document.documentElement).appendChild(player);
@@ -802,8 +813,8 @@ function loadMediaSessionsShim() {
             };
 
             player.replayAfterRemoval = () => {
-                if (player.pausedBecauseOfDomRemoval === true) {
-                    delete player.pausedBecauseOfDomRemoval;
+                if (player.dataset.pbiPausedForDomRemoval === "true") {
+                    delete player.dataset.pbiPausedForDomRemoval;
                     player.removeEventListener("pause", player.replyAfterRemoval);
 
                     player.play();
@@ -814,27 +825,39 @@ function loadMediaSessionsShim() {
             player.addEventListener("pause", player.replayAfterRemoval);
         `;
 
-        executeScript(`function() {
-                var oldCreateElement = Document.prototype.createElement;
-                Document.prototype.createElement = function() {
-                    var createdTag = oldCreateElement.apply(this, arguments);
+        const handleCreateElement = `
+            const tagName = arguments[0];
 
-                    var tagName = arguments[0];
-
-                    if (typeof tagName === "string") {
-                        if (tagName.toLowerCase() === "audio") {
-                            const player = createdTag;
-                            ${addPlayerToDomEvadingAutoPlayBlocking}
-                        } else if (tagName.toLowerCase() === "video") {
-                            (document.head || document.documentElement).appendChild(createdTag);
-                            createdTag.parentNode.removeChild(createdTag);
-                        }
-                    }
-
-                    return createdTag;
-                };
+            if (typeof tagName === "string") {
+                if (tagName.toLowerCase() === "audio") {
+                    const player = createdTag;
+                    ${addPlayerToDomEvadingAutoPlayBlocking}
+                } else if (tagName.toLowerCase() === "video") {
+                    (document.head || document.documentElement).appendChild(createdTag);
+                    createdTag.parentNode.removeChild(createdTag);
+                }
             }
-        `);
+        `;
+
+        if (IS_FIREFOX) {
+            const oldCreateElement = Document.prototype.createElement;
+            exportFunction(function() {
+                const createdTag = oldCreateElement.apply(this, arguments);
+                eval(handleCreateElement);
+                return createdTag;
+            }, Document.prototype, {defineAs: "createElement"});
+        } else {
+            executeScript(`
+                function() {
+                    const oldCreateElement = Document.prototype.createElement;
+                    Document.prototype.createElement = function() {
+                        const createdTag = oldCreateElement.apply(this, arguments);
+                        ${handleCreateElement}
+                        return createdTag;
+                    };
+                }
+            `);
+        }
 
         // We also briefly add items created as new Audio() to the DOM so we can control it
         // similar to the document.createElement hack above since we cannot share variables
@@ -846,12 +869,22 @@ function loadMediaSessionsShim() {
             // It also doesn't seem to have the aggressive autoplay prevention Chrome has,
             // so the horrible replyAfterRemoval hack from above isn't copied into this
             // See Bug 411148: Music playing from the ownCloud Music app does not show up
-            var oldAudio = window.Audio;
-            exportFunction(function(...args) {
+
+            // A function exported with exportFunction loses its prototype, leading to Bug 414512
+
+            const oldAudio = window.Audio;
+            // It is important to use the prototype on wrappedJSObject as the prototype
+            // of the content script is restricted and not accessible by the website
+            const oldAudioPrototype = window.wrappedJSObject.Audio.prototype;
+
+            const audioConstructor = function(...args) {
                 const player = new oldAudio(...args);
                 eval(addPlayerToDomEvadingAutoPlayBlocking);
                 return player;
-            }, window, {defineAs: "Audio"});
+            };
+            exportFunction(audioConstructor, window.wrappedJSObject, {defineAs: "Audio"});
+
+            window.wrappedJSObject.Audio.prototype = oldAudioPrototype;
         } else {
             executeScript(`function() {
                 var oldAudio = window.Audio;
